﻿local DebuffTypeColor = DebuffTypeColor

local _, class = UnitClass("player")
local canDispelMagic

local module = Grid:NewStatusModule("GridStatusDispellableDebuffs")
module.options = false

local GridRoster = Grid:GetModule("GridRoster")

module.defaultDB = {
	debug = false,
	alert_dispellableDebuffs = {
		text = "Dispellable debuffs",
		enable = true,
		color = {r = 0.2, g = 0.6, b = 1.0, a = 1.0},
		priority = 90,
		range = false,
	},
}

local options = {
	color = false,
}

function module:PostInitialize()
	self:RegisterStatus("alert_dispellableDebuffs", "Dispellable debuffs", options, true)
end

function module:OnStatusEnable(status)
	if status == "alert_dispellableDebuffs" then
		self:RegisterMessage("Grid_UnitJoined")
		self:RegisterEvent("UNIT_AURA", "ScanUnitAuras")
		self:RegisterEvent("PLAYER_TALENT_UPDATE")

		self:UpdateAllUnitAuras()
	end
end

function module:OnStatusDisable(status)
	if status == "alert_dispellableDebuffs" then
		self:UnregisterMessage("Grid_UnitJoined")
		self:UnregisterEvent("UNIT_AURA")
		self:UnregisterEvent("PLAYER_TALENT_UPDATE")

		self.core:SendStatusLostAllUnits(status)
	end
end

function module:Grid_UnitJoined(event, unitGUID, unitID)
	self:ScanUnitAuras(event, unitID, unitGUID)
end

function module:UpdateAllUnitAuras()
	for unitGUID, unitID in GridRoster:IterateRoster() do
		self:ScanUnitAuras("UpdateAllUnitAuras", unitID, unitGUID)
	end
end

local debuffTypePriority = {
	Magic = 1,
	Curse = 2,
	Poison = 3,
	Disease = 4,
}

function module:ScanUnitAuras(event, unitID, unitGUID)
	local guid = unitGUID or UnitGUID(unitID)

	local settings = self.db.profile.alert_dispellableDebuffs

	local debuffTypeDisp, iconDisp, countDisp, durationDisp, startDisp
	local index = 0
	while true do
		index = index + 1
		local name, _, icon, count, debuffType, duration, expirationTime = UnitDebuff(unitID, index, "RAID")

		if not name then
			self.core:SendStatusLost(guid, "alert_dispellableDebuffs")
			break
		end
		
		if (debuffType ~= "Magic" or (canDispelMagic or (UnitIsUnit(unitID, "player") and class == "PRIEST"))) and (not debuffTypeDisp or (debuffTypePriority[debuffType] < debuffTypePriority[debuffTypeDisp])) then
			-- iconDisp = "Interface\\RaidFrame\\Raid-Icon-Debuff"..debuffType
			iconDisp, countDisp, debuffTypeDisp, durationDisp, startDisp = icon, count, debuffType, duration, expirationTime - duration
		end
	end
	
	if debuffTypeDisp then
		self.core:SendStatusGained(guid, "alert_dispellableDebuffs", settings.priority, (settings.range and 40), DebuffTypeColor[debuffTypeDisp], nil, nil, nil, iconDisp, startDisp, durationDisp, countDisp)
	end
end

function module:PLAYER_TALENT_UPDATE()
	if class == "DRUID" then
		canDispelMagic = (select(5, GetTalentInfo(3, 17)) or 0) >= 1 -- Nature's Cure
	elseif class == "PALADIN" then
		canDispelMagic = (select(5, GetTalentInfo(1, 14)) or 0) >= 1 -- Sacred Cleansing
	elseif class == "PRIEST" then
		canDispelMagic = GetPrimaryTalentTree() ~= 3 -- Dispel Magic can only dispel (cure) others if you're disc or holy
	elseif class == "SHAMAN" then
		canDispelMagic = (select(5, GetTalentInfo(3, 12)) or 0) >= 1 -- Improved Cleanse Spirit
	-- elseif class == "WARLOCK" then
		-- canDispel.Magic = IsSpellKnown(89808, true) -- Singe Magic (Imp)
	end
	
	self:UpdateAllUnitAuras()
end